@@ -25,6 +25,7 @@ sub new {
             &SETTINGS_ENABLE_PUSH            => DEFAULT_ENABLE_PUSH,
             &SETTINGS_MAX_CONCURRENT_STREAMS => DEFAULT_MAX_CONCURRENT_STREAMS,
             &SETTINGS_INITIAL_WINDOW_SIZE    => DEFAULT_INITIAL_WINDOW_SIZE,
+            &SETTINGS_COMPRESS_DATA          => DEFAULT_COMPRESS_DATA,
         },
 
         streams => {},
@@ -355,7 +356,13 @@ sub send_pp_headers {
 
 sub send_data {
     my ( $self, $stream_id, $data ) = @_;
-    while ( ( my $l = length($data) ) > 0 ) {
+    my $blocked_ref = $self->stream_blocked_data($stream_id);
+    if ( defined $$blocked_ref ) {
+        $data         = $$blocked_ref . $data;
+        $$blocked_ref = undef;
+    }
+    while (1) {
+        my $l    = length($data);
         my $size = MAX_PAYLOAD_SIZE;
         for ( $l, $self->fcw_send, $self->stream_fcw_send($stream_id) ) {
             $size = $_ if $size > $_;
@@ -363,7 +370,7 @@ sub send_data {
         my $flags = $l == $size ? END_STREAM : 0;
 
         # Flow control
-        if ( $size == 0 ) {
+        if ( $l != 0 && $size == 0 ) {
             $self->stream_blocked_data( $stream_id, $data );
             last;
         }
@@ -375,6 +382,7 @@ sub send_data {
                 $stream_id, \substr( $data, 0, $size, '' )
             )
         );
+        last if $flags & END_STREAM;
     }
 }
 
@@ -385,6 +393,12 @@ sub send_blocked {
     }
 }
 
+sub issue_blocked {
+    my $self = shift;
+    $self->{issue_blocked} = shift if @_;
+    $self->{issue_blocked};
+}
+
 sub error {
     my $self = shift;
     if ( @_ && !$self->{shutdown} ) {
