@@ -15,7 +15,7 @@ use constant {
     DEFAULT_ENABLE_PUSH            => 1,
     DEFAULT_MAX_CONCURRENT_STREAMS => 100,
     DEFAULT_INITIAL_WINDOW_SIZE    => 65_535,
-    DEFAULT_SETTINGS_COMPRESS_DATA => 0,
+    DEFAULT_COMPRESS_DATA          => 0,
 
     # Priority
     DEFAULT_WEIGHT => 16,
@@ -114,7 +114,8 @@ our %EXPORT_TAGS = (
           DEFAULT_HEADER_TABLE_SIZE
           DEFAULT_MAX_CONCURRENT_STREAMS
           DEFAULT_ENABLE_PUSH
-          DEFAULT_INITIAL_WINDOW_SIZE)
+          DEFAULT_INITIAL_WINDOW_SIZE
+          DEFAULT_COMPRESS_DATA)
     ],
     states    => [qw(IDLE RESERVED OPEN HALF_CLOSED CLOSED)],
     endpoints => [qw(CLIENT SERVER)],
